/*
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Bonneau and Obeo - initial API and implementation
 */

package fr.obeo.acceleo.gen.debug.ui.actions;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.model.IBreakpoint;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.Position;
import org.eclipse.jface.text.source.IAnnotationModel;
import org.eclipse.jface.text.source.IVerticalRulerInfo;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.texteditor.AbstractMarkerAnnotationModel;
import org.eclipse.ui.texteditor.IDocumentProvider;
import org.eclipse.ui.texteditor.ITextEditor;
import org.eclipse.ui.texteditor.IUpdate;

import fr.obeo.acceleo.gen.debug.ui.AcceleoGenDebugUiPlugin;
import fr.obeo.acceleo.gen.debug.ui.model.AcceleoLineBreakpoint;
import fr.obeo.acceleo.gen.debug.ui.model.IAcceleoDebugConstants;

/**
 * The abstract breakpoint ruler action.
 * 
 * @author www.obeo.fr
 * 
 */
public abstract class AbstractBreakpointRulerAction extends Action implements IUpdate {

	/**
	 * A vertical ruler is a visual component which may serve text viewers as an
	 * annotation presentation area.
	 */
	protected IVerticalRulerInfo fInfo;

	/**
	 * The text editor.
	 */
	protected ITextEditor fTextEditor;

	/**
	 * The breakpoint.
	 */
	private IBreakpoint fBreakpoint;

	/**
	 * @return the breakpoint
	 */
	protected IBreakpoint getBreakpoint() {
		return fBreakpoint;
	}

	/**
	 * @param breakpoint
	 *            is the breakpoint
	 */
	protected void setBreakpoint(IBreakpoint breakpoint) {
		fBreakpoint = breakpoint;
	}

	/**
	 * @return the text editor
	 */
	protected ITextEditor getTextEditor() {
		return fTextEditor;
	}

	/**
	 * @param textEditor
	 *            is text editor
	 */
	protected void setTextEditor(ITextEditor textEditor) {
		fTextEditor = textEditor;
	}

	/**
	 * @return the vertical ruler
	 */
	protected IVerticalRulerInfo getInfo() {
		return fInfo;
	}

	/**
	 * @param info
	 *            is the vertical ruler
	 */
	protected void setInfo(IVerticalRulerInfo info) {
		fInfo = info;
	}

	/**
	 * Returns the breakpoint.
	 * 
	 * @return the breakpoint
	 */
	protected IBreakpoint determineBreakpoint() {
		IBreakpoint[] breakpoints = DebugPlugin.getDefault().getBreakpointManager().getBreakpoints(IAcceleoDebugConstants.ID_ACCELEO_DEBUG_MODEL);
		for (int i = 0; i < breakpoints.length; i++) {
			IBreakpoint breakpoint = breakpoints[i];
			if (breakpoint instanceof AcceleoLineBreakpoint) {
				try {
					if (breakpointAtRulerLine((AcceleoLineBreakpoint) breakpoint)) {
						return breakpoint;
					}
				} catch (CoreException e) {
					AcceleoGenDebugUiPlugin.getDefault().log(e, true);
					continue;
				}
			}
		}
		return null;
	}

	private boolean breakpointAtRulerLine(AcceleoLineBreakpoint breakpoint) throws CoreException {
		AbstractMarkerAnnotationModel model = getAnnotationModel();
		if (model != null) {
			Position position = model.getMarkerPosition(breakpoint.getMarker());
			if (position != null) {
				IDocumentProvider provider = getTextEditor().getDocumentProvider();
				IDocument doc = provider.getDocument(getTextEditor().getEditorInput());
				try {
					int markerLineNumber = doc.getLineOfOffset(position.getOffset());
					int rulerLine = getInfo().getLineOfLastMouseButtonActivity();
					if (rulerLine == markerLineNumber) {
						if (getTextEditor().isDirty()) {
							return breakpoint.getLineNumber() == markerLineNumber + 1;
						}
						return true;
					}
				} catch (BadLocationException x) {
				}
			}
		}

		return false;
	}

	/**
	 * Returns the <code>AbstractMarkerAnnotationModel</code> of the editor's
	 * input.
	 * 
	 * @return the marker annotation model
	 */
	protected AbstractMarkerAnnotationModel getAnnotationModel() {
		IDocumentProvider provider = fTextEditor.getDocumentProvider();
		IAnnotationModel model = provider.getAnnotationModel(getTextEditor().getEditorInput());
		if (model instanceof AbstractMarkerAnnotationModel) {
			return (AbstractMarkerAnnotationModel) model;
		}
		return null;
	}

	/**
	 * @return the resource for which to create the marker or <code>null</code>
	 */
	protected IResource getResource() {
		IEditorInput input = fTextEditor.getEditorInput();
		IResource resource = (IResource) input.getAdapter(IFile.class);
		if (resource == null) {
			resource = (IResource) input.getAdapter(IResource.class);
		}
		return resource;
	}

}